﻿using SharpSword;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using WideMargin.MVCInterfaces;
using WideMargin.Utilities;

namespace WideMargin.Gui.Wpf
{
    /// <summary>
    /// Interaction logic for GenericView.xaml
    /// </summary>
    public partial class GenericView : UserControl, IGenericView
    {
        public GenericView()
        {
            InitializeComponent();
            _webView.RequestVersification += WebViewRequestVersification;
            _webView.VerseLinkClicked += WebViewVerseLinkClicked;
            _webView.ActionLinkClicked += WebView_ActionLinkClicked;
            _webView.RequestMore += (sender, args) => RequestMore.Fire(this, args);
        }

        void WebView_ActionLinkClicked(object sender, LinkClickedEventArgs<string> e)
        {
            if (e.Button == 3)
            {
                Dispatcher.Invoke((Action)(() =>
                {
                    _openInNewTabAction = () =>
                    {
                        ActionLinkClicked.Fire(this, new LinkClickedEventArgs<string>(e.LinkTarget, 2));
                    };
                    _grid.ContextMenu.IsOpen = true;
                }));
                return;
            }
            ActionLinkClicked.Fire(this, e);
        }

        Action _openInNewTabAction;

        void WebViewVerseLinkClicked(object sender, LinkClickedEventArgs<IVerseIdentifier> e)
        {
            if (e.Button == 3)
            {
                Dispatcher.Invoke((Action)(() =>
                {
                    _openInNewTabAction = () =>
                    {
                        VerseLinkClicked.Fire(this, new LinkClickedEventArgs<IVerseIdentifier>(e.LinkTarget, 2));
                    };
                    _grid.ContextMenu.IsOpen = true;
                }));
                return;
            }
            VerseLinkClicked.Fire(this, e);
        }

        void OpenInNewTabMenuItem_Click(object sender, RoutedEventArgs e)
        {
            Action action = _openInNewTabAction;
            if (action != null)
            {
                action();
            }
        }

        void WebViewRequestVersification(object sender, RequestVersificationEventArgs e)
        {
            RequestVersification.Fire(this, e);
        }

        public void AppendText(string text)
        {
            _webView.AppendText(text);
        }

        public string Text
        {
            set 
            {
                Dispatcher.Invoke((Action)(() =>
                {
                    _webView.Text = value;
                }));
            }
        }

        internal void ExecuteScript(string script, params object[] args)
        {
            _webView.ExecuteScript(script, args);
        }

        public event EventHandler<LinkClickedEventArgs<SharpSword.IVerseIdentifier>> VerseLinkClicked;

        public event EventHandler<LinkClickedEventArgs<string>> ActionLinkClicked;

        public event EventHandler<RequestVersificationEventArgs> RequestVersification;

        public event EventHandler<EventArgs> RequestMore;
    }
}
