﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using WideMargin.MVCInterfaces;
using WideMargin.Utilities;

namespace WideMargin.Gui.Wpf
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        TabItem _newTab;

        public MainWindow()
        {
            InitializeComponent();
            _tabControl.SelectionChanged += TabControl_SelectionChanged;

            _newTab = new TabItem();
            _newTab.Header = "+";
            _tabControl.Items.Add(_newTab);
        }

        internal IBibleView AddTab()
        {
            BibleView bibleView = null;
            Dispatcher.Invoke((Action)(() =>
            {
                bibleView = new BibleView();
                bibleView.TabCloseRequested += BibleView_TabCloseRequested;
                var tabItem = new TabItem();
                bibleView.TabPage = tabItem;
                tabItem.Header = bibleView.CreateTabTitle();
                tabItem.Content = bibleView;
                int newTabIndex = _tabControl.Items.Count - 1;
                _tabControl.Items.Insert(newTabIndex, tabItem);
                _tabControl.SelectedIndex = newTabIndex;
                _tabControl.Padding = new Thickness(0);
            }));

            return bibleView;
        }

        internal void AddNewTabTrigger()
        {
            Dispatcher.Invoke((Action)(() =>
            {
                _tabControl.Items.Add(_newTab);
            }));
        }

        internal void RemoveNewTabTrigger()
        {
            Dispatcher.Invoke((Action)(() =>
            {
                _tabControl.Items.Remove(_newTab);
            }));
        }

        void BibleView_TabCloseRequested(object sender, EventArgs e)
        {
            var page = (sender as BibleView).TabPage;
            int pageIndex = _tabControl.Items.IndexOf(page);
            int numberOfPages = _tabControl.Items.Count;
            //if second to last page removing it will activate add page
            //causing a new page to be added.
            if (pageIndex == numberOfPages - 2)
            {
                if (numberOfPages - 3 >= 0)
                {
                    _tabControl.SelectedIndex = numberOfPages - 3;
                }
            }

            numberOfPages--;

            if (numberOfPages <= 1)
            {
                Application.Current.Shutdown();
                return;
            }

            _tabControl.Items.Remove(page);

        }

        internal event EventHandler<EventArgs> NewTabRequested;

        void TabControl_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0)
            {
                return;
            }
            if (e.AddedItems[0] == _newTab)
            {
                NewTabRequested.Fire(this, EventArgs.Empty);
            }
        }
    }
}
